# -*- coding: utf-8 -*-
from .base import BaseEntity


class Item(BaseEntity):
    """
    |     Nome     |           Descrição            |     Tipo    |    Tamanho    | Obrigatório |
    |==============|================================|=============|===============|=============|
    | ID           | Código do Produto              | Texto       | 50            | S           |
    | Name         | Nome do Produto                | Texto       | 150           | S           |
    | ItemValue    | Valor Unitário                 | Número      | (20,4)        | S           |
    | Qty          | Quantidade                     | Número      | (1,0)         | S           |
    | Gift         | Presente                       | Número      | (1,0)         | N           |
    | CategoryID   | Código da Categoria do Produto | Número      | (1,0)         | N           |
    | CategoryName | Nome da Categoria do Produto   | Texto       | 200           | N           |

    - XML representação:
        <Item>
            <ID></ID>
            <Name></Name>
            <ItemValue></ItemValue>
            <Qty></Qty>
            <Gift></Gift>
            <CategoryID></CategoryID>
            <CategoryName></CategoryName>
        </Item>
    """

    def __init__(
                    self,
                    ID,
                    Name,
                    ItemValue,
                    Qty,
                    Gift=None,
                    CategoryID=None,
                    CategoryName=None
                ):
        self._data = {}
        self._data["Item"] = {
            "ID": ID,
            "Name": Name,
            "ItemValue": ItemValue,
            "Qty": Qty
        }

        self.set_no_mandatory_fields_values(locals())

    def get_no_mandatory_fields(self):
        return (
            "Gift", "CategoryID", "CategoryName"
        )



# -*- coding: utf-8 -*-
from .base import BaseEntity


class FingerPrint(BaseEntity):
    """
    |    Nome   |                Descrição                 |  Tipo  | Tamanho  | Obrigatório |
    |===========|==========================================|========|==========|=============|
    | SessionID | Identificador único da sessão do usuário | String |      128 | S           |

    <FingerPrint>
        <SessionID></SessionID>
    </FingerPrint>
    """

    def __init__(self, SessionID):
        self._data = {}
        self._data["FingerPrint"] = {"SessionID": SessionID}


class Address(BaseEntity):
    """
    |     Nome    |                 Descrição                 |     Tipo    |    Tamanho    | Obrigatório |
    |=============|===========================================|=============|===============|=============|
    | Street      | Nome do logradouro (Sem abreviações)      | Texto       |           200 | S           |
    | Number      | Número do Endereço                        | Texto       |            15 | S           |
    | Comp        | Complemento do Endereço (Sem abreviações) | Texto       |           250 | N           |
    | County      | Bairro do Endereço (Sem abreviações)      | Texto       |           150 | S           |
    | City        | Cidade do Endereço (Sem abreviações)      | Texto       |           150 | S           |
    | State       | Sigla do Estado do Endereço - UF          | Texto       |             2 | S           |
    | Country     | Pais do Endereço (Sem abreviações)        | Texto       |           150 | N           |
    | ZipCode     | CEP do Endereço                           | Número      |            10 | S           |
    | Reference   | Referência do Endereço (Sem abreviações)  | Texto       |           250 | N           |

    - XML representação:
        <Address>
            <Street></Street>
            <Number></Number>
            <Comp></Comp>
            <County></County>
            <City></City>
            <State></State>
            <Country></Country>
            <ZipCode></ZipCode>
            <Reference></Reference>
        </Address>
    """
    def __init__(
                    self,
                    Street,
                    Number,
                    County,
                    City,
                    State,
                    ZipCode,
                    Country=None,
                    Comp=None,
                    Reference=None
                ):
        self._data = {}
        self._data["Address"] = {
            "Street": Street,
            "Number": Number,
            "County": County,
            "City": City,
            "State": State,
            "ZipCode": ZipCode,
        }

        self.set_no_mandatory_fields_values(locals())

    def get_no_mandatory_fields(self):
        return (
            "Country", "Comp", "Reference"
        )


class Phone(BaseEntity):
    """
    |    Nome   |                  Descrição                   |   Tipo   |  Tamanho   | Obrigatório |
    |===========|==============================================|==========|============|=============|
    | Type      | Tipo de Telefone (Lista de Tipo de Telefone) | Número   |          1 | S           |
    | DDI       | DDI do Telefone                              | Número   |          3 | N           |
    | DDD       | DDD do Telefone                              | Número   |          2 | S           |
    | Number    | Número do Telefone                           | Número   |          9 | S           |
    | Extension | Ramal do Telefone                            | Texto    |         10 | N           |

    - XML representação:
        <Phone>
            <Type></Type>
            <DDI></DDI>
            <DDD></DDD>
            <Number></Number>
            <Extension></Extension>
        </Phone>
    """

    NAO_DEFINIDO = 0
    RESIDENCIAL = 1
    COMERCIAL = 2
    RECADOS = 3
    COBRANCA = 4
    TEMPORARIO = 5
    CELULAR = 6
    TYPES = (
        NAO_DEFINIDO,
        RESIDENCIAL,
        COMERCIAL,
        RECADOS,
        COBRANCA,
        TEMPORARIO,
        CELULAR,
    )

    def __init__(self, Type, DDD, Number, DDI=None, Extension=None):
        self._data = {}
        self._data["Phone"] = {
            "Type": Type,
            "DDD": DDD,
            "Number": Number,
        }

        self.set_no_mandatory_fields_values(locals())

    def get_no_mandatory_fields(self):
        return (
            "DDI", "Extension"
        )



# -*- coding: utf-8 -*-
from .base import BaseEntity


class Payment(BaseEntity):
    """
    |        Nome        |                     Descrição                     |        Tipo       |       Tamanho        | Obrigatório |
    |====================|===================================================|===================|======================|=============|
    | Sequential         | Sequencia de realização do pagamento              | Número            | (1,0)                | N           |
    | Date               | Data do pagamento                                 | Data              | (yyyy-mmddThh:mm:ss) | S           |
    | Amount             | Valor cobrado neste pagamento                     | Número            | (20,4)               | S           |
    | PaymentTypeID      | Tipo de Pagamento (Lista de Tipos de Pagamento)   | Número            | (2,0)                | S           |
    | Address            | instância de Address                              | Address instância |                      | S           |
    | QtyInstallments    | Quantidade de Parcelas                            | Número            | (2,0)                | N           |
    | Interest           | Taxa de Juros                                     | Número            | (4,2)                | N           |
    | InterestValue      | Valor dos Juros                                   | Número            | (20,4)               | N           |
    | CardNumber         | Número do Cartão                                  | Texto             | 200                  | N           |
    | CardBin            | Número do BIN do Cartão                           | Texto             | 6                    | N           |
    | CardEndNumber      | 4 últimos digitos do número de cartão             | Texto             | 4                    | N           |
    | CardType           | Bandeira do Cartão (Lista de Bandeiras de Cartão) | Número            | (1,0)                | N           |
    | CardExpirationDate | Data da Expiração                                 | Texto             | 50                   | N           |
    | Name               | Nome de Cobrança                                  | Texto             | 150                  | N           |
    | LegalDocument      | Documento da Pessoa de Cobrança                   | Texto             | 100                  | N           |
    |                    |                                                   |                   |                      |             |

    - XML representação:
        <Payment>
            <Sequential></Sequential>
            <Date></Date>
            <Amount></Amount>
            <PaymentTypeID></PaymentTypeID>
            <QtyInstallments></QtyInstallments>
            <Interest></Interest>
            <InterestValue></InterestValue>
            <CardNumber></CardNumber>
            <CardBin></CardBin>
            <CardEndNumber></CardEndNumber>
            <CardType></CardType>
            <CardExpirationDate></CardExpirationDate>
            <Name></Name>
            <LegalDocument></LegalDocument>
            <Address>
                <Street></Street>
                <Number></Number>
                <Comp></Comp>
                <County></County>
                <City></City>
                <State></State>
                <Country></Country>
                <ZipCode></ZipCode>
            </Address>
            <Nsu></Nsu>
            <Currency></Currency>
        </Payment>
    """

    CARTAO_CREDITO = 1
    BOLETO_BANCARIO = 2
    DEBITO_BANCARIO = 3
    DEBITO_BANCARIO_DINHEIRO = 4
    DEBITO_BANCARIO_CHEQUE = 5
    TRANSFERENCIA_BANCARIA = 6
    SEDEX_A_COBRAR = 7
    CHEQUE = 8
    DINHEIRO = 9
    FINANCIAMENTO = 10
    FATURA = 11
    CUPOM = 12
    MULTICHEQUE = 13
    OUTROS = 14
    PAYMENT_TYPES_ID = (
        CARTAO_CREDITO,
        BOLETO_BANCARIO,
        DEBITO_BANCARIO,
        DEBITO_BANCARIO_DINHEIRO,
        DEBITO_BANCARIO_CHEQUE,
        TRANSFERENCIA_BANCARIA,
        SEDEX_A_COBRAR,
        CHEQUE,
        DINHEIRO,
        FINANCIAMENTO,
        FATURA,
        CUPOM,
        MULTICHEQUE,
        OUTROS,
    )

    def __init__(
                    self,
                    Date,
                    Amount,
                    PaymentTypeID,
                    Address=None,
                    Sequential=None,
                    QtyInstallments=None,
                    Interest=None,
                    InterestValue=None,
                    CardNumber=None,
                    CardBin=None,
                    CardEndNumber=None,
                    CardType=None,
                    CardExpirationDate=None,
                    Name=None,
                    LegalDocument=None,
                    Nsu=None,
                    Currency=None,
                ):
        self._data = {}
        self._data["Payment"] = {
            "Date": Date,
            "Amount": Amount,
            "PaymentTypeID": PaymentTypeID,
        }

        if Address:
            self._data["Payment"].update(Address.get_dict())

        self.set_no_mandatory_fields_values(locals())

    def get_no_mandatory_fields(self):
        return (
            "Sequential", "QtyInstallments", "Interest", "InterestValue",
            "CardNumber", "CardBin", "CardEndNumber", "CardType",
            "CardExpirationDate", "Name", "LegalDocument", "Nsu", "Currency"
        )



# -*- coding: utf-8 -*-
from .base import BaseCustomer


class BillingData(BaseCustomer):
    """
    |    Nome   |     Descrição      | Tipo |       Tamanho        | Obrigatório |
    |===========|====================|======|======================|=============|
    | BirthDate | Data de Nascimento | Data | (yyyy-mmddThh:mm:ss) | S           |
    """

    def __init__(self, BirthDate, *args, **kwargs):
        kwargs["BirthDate"] = BirthDate
        super(BillingData, self).__init__(*args, **kwargs)
        self._data_temp = self._data
        self._data = {}
        self._data["BillingData"] = self._data_temp



# -*- coding: utf-8 -*-
from suds.client import Client
from .responses import OrderStatusResponse, SendOrdersResponse


class ClearSaleConnector():
    def __init__(self, entity_code, use_sandbox=True, **extra_params_connection):
        self._entity_code = entity_code
        ws_url = "http://homologacao.clearsale.com.br/integracaov2/service.asmx?wsdl" if use_sandbox \
            else "http://integracao.clearsale.com.br/service.asmx?wsdl"
        # print "====== extra_params_connection ======"
        # print extra_params_connection
        self._ws_client = Client(ws_url, **extra_params_connection)

    def get_ws_client(self):
        return self._ws_client

    def get_entity_code(self):
        return self._entity_code


class ClearSaleService():
    def __init__(self, clearsale_connector, *args, **kwargs):
        self._connector = clearsale_connector

    def send_orders(self, orders):
        xml = u"<ClearSale>{0}</ClearSale>".format(orders.get_xml())
        xml_ret = self._connector.get_ws_client().service.SendOrders(
            self._connector.get_entity_code(), xml)
        return SendOrdersResponse(xml_ret.format())

    def get_order_status(self, order_id):
        xml_ret = self._connector.get_ws_client().service.GetOrderStatus(
            self._connector.get_entity_code(), order_id)
        return OrderStatusResponse(xml_ret.format())

    def get_connector(self):
        return self._connector

    def set_connector(self, clearsale_connector):
        self._connector = clearsale_connector



# -*- coding: utf-8 -*-
import json
from xml.dom.minidom import parseString


class BaseEntity(object):

    def serialize(self, root):
        """
        http://www.madchicken.it/2010/06/serialize-a-python-dictionary-to-xml.html
        """
        xml = ''
        for key in root.keys():
            # print "=========="
            # print key
            if isinstance(root[key], dict):
                xml = u'{}<{}>{}</{}>'.format(xml, key, self.serialize(root[key]), key)
            elif isinstance(root[key], list):
                xml = u'{}<{}>'.format(xml, key)
                for item in root[key]:
                    xml = u'{}{}'.format(xml, self.serialize(item))
                xml = u'{}</{}>'.format(xml, key)
            else:
                value = root[key]
                xml = u'{}<{}>{}</{}>'.format(xml, key, value, key)
        return xml

    def get_data(self):
        return self._data

    def get_dict(self):
        data_temp = self._data
        # for k in self._data:
        #     if isinstance(data_temp[k], list):
        #         for i, item in enumerate(data_temp[k]):
        #             if isinstance(item, BaseEntity):
        #                 data_temp[k][i] = item.get_dict()
        #     elif isinstance(data_temp[k], BaseEntity):
        #         data_temp[k] = data_temp[k].get_dict()
        return data_temp

    def get_pretty_dict(self):
        return json.dumps(self.get_dict(), indent=4,)

    def get_xml_root(self):
        return None

    def get_xml(self):
        # root = self.get_xml_root()
        # params = {"root": False,
        #           "attr_type": False}

        # xml = dicttoxml.dicttoxml(self.get_dict(), **params)
        xml = self.serialize(self.get_dict())
        # if root is not None:
        #     xml = "<{0}>{1}</{0}>".format(root, xml)
        return xml

    def get_pretty_xml(self):
        dom = parseString(self.get_xml())
        return dom.toprettyxml()

    def get_no_mandatory_fields(self):
        return []

    def set_no_mandatory_fields_values(self, extra_fields):
        if not self._data:
            self._data = {}

        keys = self._data.keys()
        if keys:
            if isinstance(self._data[keys[0]], dict):
                data_temp = self._data[keys[0]]
            else:
                data_temp = self._data

            for field in self.get_no_mandatory_fields():
                # print field
                value = extra_fields[field]
                # print value
                # print "======================="
                if value is not None:
                    data_temp[field] = value


class BaseCustomer(BaseEntity):
    """
    |      Nome      |                      Descrição                       |             Tipo             |       Tamanho        | Obrigatório |
    |================|======================================================|==============================|======================|=============|
    | ID             | Código do cliente                                    | Texto                        | 50                   | S           |
    | Type           | Pessoa Física ou Jurídica (Lista de Tipos de Pessoa) | Número                       | (1,0)                | S           |
    | LegalDocument1 | CPF ou CNPJ                                          | Texto                        | 100                  | S           |
    | LegalDocument2 | RG ou Inscrição Estadual                             | Texto                        | 100                  | N           |
    | Name           | Nome do cliente                                      | Texto                        | 500                  | S           |
    | BirthDate      | Data de Nascimento                                   | Data                         | (yyyy-mmddThh:mm:ss) | S/N         |
    | Email          | Email                                                | Texto                        | 150                  | N           |
    | Gender         | Sexo (Lista de Tipo de Sexo)                         | Texto                        | 1                    | N           |
    | Address        | Instância de BillingDataAddress                      | BillingDataAddress instância |                      | S           |

    - XML representação:
        <BillingData> ou <ShippingData>
            <ID></ID>
            <Type></Type>
            <LegalDocument1></LegalDocument1>
            <LegalDocument2></LegalDocument2>
            <Name></Name>
            <BirthDate></BirthDate>
            <Email></Email>
            <Gender></Gender>
            <Address>
                <Street></Street>
                <Number></Number>
                <Comp></Comp>
                <County></County>
                <City></City>
                <State></State>
                <Country></Country>
                <ZipCode></ZipCode>
                <Reference></Reference>
            </Address>
            <Phones>
                <Phone>
                    <Type></Type>
                    <DDI></DDI>
                    <DDD></DDD>
                    <Number></Number>
                    <Extension></Extension>
                </Phone>
            </Phones>
        </BillingData> ou </ShippingData>
    """

    TYPE_PESSOA_FISICA = 1
    TYPE_PESSOA_JURIDICA = 2

    GENDER_MASCULINO = 'M'
    GENDER_FEMININO = 'F'

    def __init__(
                    self,
                    ID,
                    Type,
                    LegalDocument1,
                    Name,
                    Address,
                    BirthDate=None,
                    LegalDocument2=None,
                    Email=None,
                    Gender=None
                ):
        self._data = {
            "ID": ID,
            "Type": Type,
            "LegalDocument1": LegalDocument1,
            "Name": Name,
            "Phones": [],
        }

        self._data.update(Address.get_dict())

        self.set_no_mandatory_fields_values(locals())

    def get_no_mandatory_fields(self):
        return (
            "LegalDocument2", "Email", "Gender", "BirthDate"
        )

    def add_phone(self, Phone):
        self._data[self._data.keys()[0]]["Phones"].append(Phone.get_dict())



# -*- coding: utf-8 -*-
from .base import BaseCustomer


class ShippingData(BaseCustomer):
    """
    |    Nome   |     Descrição      | Tipo |       Tamanho        | Obrigatório |
    |===========|====================|======|======================|=============|
    | BirthDate | Data de Nascimento | Data | (yyyy-mmddThh:mm:ss) | N           |
    """
    def __init__(self, *args, **kwargs):
        super(ShippingData, self).__init__(*args, **kwargs)
        self._data_temp = self._data
        self._data = {}
        self._data["ShippingData"] = self._data_temp



# -*- coding: utf-8 -*-
from .base import BaseEntity


class Orders(BaseEntity):
    def __init__(self, *args, **kwargs):
        self._data = {"Orders": []}

    def add_order(self, order):
        self._data["Orders"].append(order.get_dict())


class Order(BaseEntity):
    """
    |        Nome          |                                    Descrição                                       |          Tipo             | Tamanho   |                                             Obrigatório                                                |
    | ==================== | ================================================================================== | ========================  | ========= | ====================================================================================================== |
    | ID                   | Código do pedido                                                                   | String                    | 50        | S                                                                                                      |
    | FingerPrint          | Instância de FingerPrint                                                           | FingerPrint instância     |           | S                                                                                                      |
    | Date                 | Data do pedido                                                                     | Data (yyyy-mmddThh:mm:ss) |           | S                                                                                                      |
    | Email                | Email do pedido                                                                    | Texto                     | 150       | S                                                                                                      |
    | TotalItems           | Valor do Itens                                                                     | Decimal                   | (20,4)    | S                                                                                                      |
    | TotalOrder           | Valor do Itens                                                                     | Decimal                   | (20,4)    | S                                                                                                      |
    | QtyInstallments      | Quantidade de Parcelas                                                             | Número                    | (1,0)     | S                                                                                                      |
    | IP                   | IP do Pedido                                                                       | Texto                     | (50)      | S                                                                                                      |
    | Status               | Status do Pedido (na entrada) (Lista de status (de entrada)                        | Número                    | (1,0)     | N (se não for enviada a tag o pedido entrará como novo )                                               |
    | Origin               | Origem do Pedido                                                                   | Texto                     | 150       | S                                                                                                      |
    | ListTypeID           | ID do tipo de lista (Lista de Tipos de Lista)                                      | Número                    | (1,0)     | S                                                                                                      |
    | ListID               | ID da lista na loja                                                                | Texto                     | 200       | S                                                                                                      |
    | BillingData          | instância de BilingData                                                            | BillingData instância     |           | S                                                                                                      |
    | ShippingData         | instância de ShoppinData                                                           | ShippingData instância    |           | S                                                                                                      |
    | B2B_B2C=3            | Tipo do ecommerce                                                                  | Texto                     | 3         | N                                                                                                      |
    | ShippingPrice=""     | Valor do Frete                                                                     | Decimal                   | (20,4)    | N                                                                                                      |
    | DeliveryTimeCD=""    | Prazo de Entrega                                                                   | Texto                     | 50        | N                                                                                                      |
    | QtyItems=""          | Quantidade de Itens                                                                | Número                    | (1,0)     | N                                                                                                      |
    | QtyPaymentTypes=""   | Quantidade de Pagamentos                                                           | Número                    | (1,0)     | N                                                                                                      |
    | ShippingType=""      |                                                                                    |                           |           |                                                                                                        |
    | Gift=""              | Identifica se o pedido é presente                                                  | Número                    | (1,0)     | N                                                                                                      |
    | GiftMessage=""       | Mensagem de Presente   N                                                           | Texto                     | 8000      | N                                                                                                      |
    | Obs="",              | Observação do pedido                                                               | Texto                     | 8000      | N                                                                                                      |
    | Reanalise=0          | Marcação que indica se o pedido será reanalisado ou não (1 caso for, 0 caso não)   | Número                    | (1,0)     | N                                                                                                      |
    | Country="",          | Nome do País                                                                       | Texto                     | 50        | N(somente para pedidos de análise internacional)                                                       |
    | Nationality="",      | Nome da Nacionalidade                                                              | Texto                     | 50        | N(somente para pedidos de análise internacional)                                                       |
    | Product=4,           | ID do produto (Lista de Produtos)                                                  | Número                    | 4         | N(somente para pedidos de análise internacional ou clientes que utilizam mais produtos da ClearSale)   |
    |                      |                                                                                    |                           |           |                                                                                                        |

    - XML representação:
        <Order>
            <ID></ID>
            <FingerPrint>
                <SessionID></SessionID>
            </FingerPrint>
            <Date></Date>
            <Email></Email>
            <B2B_B2C></B2B_B2C>
            <ShippingPrice></ShippingPrice>
            <TotalItems></TotalItems>
            <TotalOrder></TotalOrder>
            <QtyInstallments></QtyInstallments>
            <DeliveryTimeCD></DeliveryTimeCD>
            <QtyItems></QtyItems>
            <QtyPaymentTypes></QtyPaymentTypes>
            <IP></IP>
            <ShippingType></ShippingType>
            <Gift></Gift>
            <GiftMessage></GiftMessage>
            <Obs></Obs>
            <Status></Status>
            <Reanalise></Reanalise>
            <Origin></Origin>
            <Country></Country>
            <Nationality></Nationality>
            <Product></Product>
            <ListTypeID></ListTypeID>
            <ListID></ListID>
            <BillingData>
                ...
            </BillingData>
            <ShippingData>
                ...
            </ShippingData>
            <Payments>
                <Payment>
                    ...
                </Payment>
            </Payments>
            <Items>
                <Item>
                    ...
                 </Item>
            </Items>
        </Order>
    """

    DATE_TIME_FORMAT = 'Y-m-d\TH:i:s'

    # B2B_B2C
    ECOMMERCE_B2B = 'b2b'
    ECOMMERCE_B2C = 'b2c'

    # Status
    STATUS_NOVO = 0
    STATUS_APROVADO = 9
    STATUS_CANCELADO = 41
    STATUS_REPROVADO = 45

    # Product
    PRODUCT_A_CLEAR_SALE = 1
    PRODUCT_M_CLEAR_SALE = 2
    PRODUCT_T_CLEAR_SALE = 3
    PRODUCT_TG_CLEAR_SALE = 4
    PRODUCT_TH_CLEAR_SALE = 5
    PRODUCT_TG_LIGHT_CLEAR_SALE = 6
    PRODUCT_TG_FULL_CLEAR_SALE = 7
    PRODUCT_T_MONITORADO = 8
    PRODUCT_SCORE_DE_FRAUDE = 9
    PRODUCT_CLEAR_ID = 10
    PRODUCT_ANALISE_INTERNACIONAL = 11

    # ListTypeID
    LIST_TYPE_NAO_CADASTRADA = 1
    LIST_TYPE_CHA_DE_BEBE = 2
    LIST_TYPE_CASAMENTO = 3
    LIST_TYPE_DESEJOS = 4
    LIST_TYPE_ANIVERSARIO = 5
    LIST_TYPE_CHA_BAR_OU_CHA_PANELA = 6

    def __init__(
                    self,
                    ID,
                    FingerPrint,
                    Date,
                    Email,
                    TotalItems,
                    TotalOrder,
                    QtyInstallments,
                    IP,
                    BillingData,
                    ShippingData,
                    Origin=None,
                    ListTypeID=None,
                    ListID=None,
                    Status=None,
                    B2B_B2C=None,
                    ShippingPrice=None,
                    DeliveryTimeCD=None,
                    QtyItems=None,
                    QtyPaymentTypes=None,
                    ShippingType=None,
                    Gift=None,
                    GiftMessage=None,
                    Obs=None,
                    Reanalise=None,
                    Country=None,
                    Nationality=None,
                    Product=None,
                ):

        self._data = {}
        self._data["Order"] = {
            "ID": ID,
            "Date": Date,
            "Email": Email,
            "TotalItems": TotalItems,
            "TotalOrder": TotalOrder,
            "QtyInstallments": QtyInstallments,
            "IP": IP,
            "Origin": Origin or "WEB",
            "Payments": [],
            "Items": [],
        }

        self._data["Order"].update(FingerPrint.get_dict())
        self._data["Order"].update(BillingData.get_dict())
        self._data["Order"].update(ShippingData.get_dict())

        self.set_no_mandatory_fields_values(locals())

    def get_no_mandatory_fields(self):
        return (
            "B2B_B2C", "ShippingPrice", "DeliveryTimeCD", "QtyItems", "QtyPaymentTypes",
            "ShippingType", "Gift", "GiftMessage", "Obs", "Reanalise", "Country",
            "Nationality", "Product", "Status", "ListTypeID", "ListID",
        )

    def add_payment(self, Payment):
        self._data["Order"]["Payments"].append(Payment.get_dict())

    def add_item(self, Item):
        self._data["Order"]["Items"].append(Item.get_dict())



# -*- coding: utf-8 -*-
import xmltodict
from xml.dom.minidom import parseString


class OrderReturn():
    # Pedido foi aprovado automaticamente segundo parâmetros definidos na regra de aprovação automática.
    STATUS_SAIDA_APROVACAO_AUTOMATICA = 'APA'
    # Pedido aprovado manualmente por tomada de decisão de um analista.
    STATUS_SAIDA_APROVACAO_MANUAL = 'APM'
    # Pedido Reprovado sem Suspeita por falta de contato com o cliente dentro do período acordado e/ou políticas restritivas de CPF (Irregular, SUS ou Cancelados).
    STATUS_SAIDA_REPROVADA_SEM_SUSPEITA = 'RPM'
    # Pedido está em fila para análise
    STATUS_SAIDA_ANALISE_MANUAL = 'AMA'
    # Ocorreu um erro na integração do pedido, sendo necessário analisar um possível erro no XML enviado e após a correção reenvia-lo.
    STATUS_SAIDA_ERRO = 'ERR'
    # Pedido importado e não classificado Score pela analisadora (processo que roda o Score de cada pedido).
    STATUS_SAIDA_NOVO = 'NVO'
    # Pedido Suspenso por suspeita de fraude baseado no contato com o “cliente” ou ainda na base ClearSale.
    STATUS_SAIDA_SUSPENSAO_MANUAL = 'SUS'
    # Cancelado por solicitação do cliente ou duplicidade do pedido.
    STATUS_SAIDA_CANCELADO_PELO_CLIENTE = 'CAN'
    # Pedido imputado como Fraude Confirmada por contato com a administradora de cartão e/ou contato com titular do cartão ou CPF do cadastro que desconhecem a compra.
    STATUS_SAIDA_FRAUDE_CONFIRMADA = 'FRD'
    # Pedido Reprovado Automaticamente por algum tipo de Regra de Negócio que necessite aplicá-la (Obs: não usual e não recomendado).
    STATUS_SAIDA_REPROVACAO_AUTOMATICA = 'RPA'
    # Pedido reprovado automaticamente por política estabelecida pelo cliente ou ClearSale.
    STATUS_SAIDA_REPROVACAO_POR_POLITICA = 'RPP'

    STATUS_APPROVED_LIST = (
        STATUS_SAIDA_APROVACAO_AUTOMATICA,
        STATUS_SAIDA_APROVACAO_MANUAL
    )

    STATUS_NOT_APPROVED_LIST = (
        STATUS_SAIDA_REPROVADA_SEM_SUSPEITA,
        STATUS_SAIDA_SUSPENSAO_MANUAL,
        STATUS_SAIDA_CANCELADO_PELO_CLIENTE,
        STATUS_SAIDA_FRAUDE_CONFIRMADA,
        STATUS_SAIDA_REPROVACAO_AUTOMATICA,
        STATUS_SAIDA_REPROVACAO_POR_POLITICA,
    )

    STATUS_WAITING_FOR_APPROVAL_LIST = (
        STATUS_SAIDA_ANALISE_MANUAL,
        STATUS_SAIDA_NOVO,
    )

    STATUS_ERROS_LIST = (
        STATUS_SAIDA_ERRO,
    )

    STATUS_LABEL = {
        STATUS_SAIDA_APROVACAO_AUTOMATICA: u"(Aprovação Automática) – Pedido foi aprovado automaticamente segundo parâmetros definidos na regra de aprovação automática.",
        STATUS_SAIDA_APROVACAO_MANUAL: u"(Aprovação Manual) – Pedido aprovado manualmente por tomada de decisão de um analista.",
        STATUS_SAIDA_REPROVADA_SEM_SUSPEITA: u"(Reprovado Sem Suspeita) – Pedido Reprovado sem Suspeita por falta de contato com o cliente dentro do período acordado e/ou políticas restritivas de CPF (Irregular, SUS ou Cancelados)",
        STATUS_SAIDA_ANALISE_MANUAL: u"(Análise manual) – Pedido está em fila para análise",
        STATUS_SAIDA_ERRO: u"(Erro) - Ocorreu um erro na integração do pedido, sendo necessário analisar um possível erro no XML enviado e após a correção reenvia-lo.",
        STATUS_SAIDA_NOVO: u"(Novo) – Pedido importado e não classificado Score pela analisadora (processo que roda o Score de cada pedido).",
        STATUS_SAIDA_SUSPENSAO_MANUAL: u"(Suspensão Manual) – Pedido Suspenso por suspeita de fraude baseado no contato com o 'cliente' ou ainda na base ClearSale.",
        STATUS_SAIDA_CANCELADO_PELO_CLIENTE: u"(Cancelado pelo Cliente) – Cancelado por solicitação do cliente ou duplicidade do pedido.",
        STATUS_SAIDA_FRAUDE_CONFIRMADA: u"(Fraude Confirmada) – Pedido imputado como Fraude Confirmada por contato com a administradora de cartão e/ou contato com titular do cartão ou CPF do cadastro que desconhecem a compra.",
        STATUS_SAIDA_REPROVACAO_AUTOMATICA: u"(Reprovação Automática) – Pedido Reprovado Automaticamente por algum tipo de Regra de Negócio que necessite aplicá-la (Obs: não usual e não recomendado).",
        STATUS_SAIDA_REPROVACAO_POR_POLITICA: u"(Reprovação Por Política) – Pedido reprovado automaticamente por política estabelecida pelo cliente ou ClearSale.",
    }

    APROVADO = 1
    AGUARDANDO_APROVACAO = 2
    REPROVADO = 3
    ERRO = 4

    STATUS = {}
    STATUS[APROVADO] = u'Aprovado'
    STATUS[AGUARDANDO_APROVACAO] = u'Aguardando aprovação'
    STATUS[REPROVADO] = u'Reprovado'
    STATUS[ERRO] = u'Erro'

    def __init__(self, id, status, score, *args, **kwargs):
        self._id = id
        self._status = status
        self._score = score

    def getID(self):
        return self._id

    def getStatus(self):
        return self._status

    def getScore(self):
        return self._score

    def approved(self):
        return self._status in self.STATUS_APPROVED_LIST

    def not_approved(self):
        return self._status in self.STATUS_NOT_APPROVED_LIST

    def waiting_for_approval(self):
        return self._status in self.STATUS_WAITING_FOR_APPROVAL_LIST

    def has_error(self):
        return self._status in self.STATUS_ERROS_LIST

    def get_analysis(self):
        if self.approved():
            return self.APROVADO
        elif self.not_approved():
            return self.REPROVADO
        elif self.waiting_for_approval():
            return self.AGUARDANDO_APROVACAO
        else:
            return self.ERRO

    def get_analysis_label(self):
        return self.STATUS[self.get_analysis()]

    def get_status_label(self):
        return self.STATUS_LABEL[self._status]


class BaseResponse():
    STATUS_CODE_TRANSACAO_CONCLUIDA = '00'
    STATUS_CODE_USUARIO_INEXISTENTE = '01'
    STATUS_CODE_ERRO_VALIDACAO_XML = '02'
    STATUS_CODE_ERRO_TRANFORMACAO_XML = '03'
    STATUS_CODE_ERRO_INESPERADO = '04'
    STATUS_CODE_PEDIDO_JA_ENVIADO_OU_NAO_ESTA_EM_REANALISE = '05'
    STATUS_CODE_ERRO_PLUGIN_ENTRADA = '06'
    STATUS_CODE_ERRO_PLUGIN_SAIDA = '07'

    def __init__(self, xml, *args, **kwargs):
        self._dict = self._parse_xml_to_dict(xml)
        self._xml = xml

        self._order = None
        self._transaction_id = None
        self._status_code = None
        self._message = None

        # print "### ======================= ###"
        # print self.__class__.__name__
        # print self._xml

        root_node = self._dict["ClearSale"] if "ClearSale" in self._dict else self._dict["PackageStatus"]

        self._orders = []
        if "Orders" in root_node:
            orders = root_node["Orders"]["Order"]
            if isinstance(orders, list):
                for o in orders:
                    if "ID" in o:
                        self._orders.append(OrderReturn(o["ID"], o["Status"], o["Score"]))
            else:
                if "ID" in orders:
                    self._orders.append(OrderReturn(orders["ID"], orders["Status"], orders["Score"]))

        if "StatusCode" in root_node:
            self._transaction_id = root_node["TransactionID"]
            self._status_code = root_node["StatusCode"]
            self._message = root_node["Message"]

    def _parse_xml_to_dict(self, xml):
        xml = xmltodict.parse(xml)
        return xml

    def getTransactionID(self):
        return self._transaction_id

    def getStatusCode(self):
        return self._status_code

    def getMessage(self):
        return self._message

    def getOrders(self):
        return self._orders

    def get_dict(self):
        return self._dict

    def get_xml(self):
        return self._xml

    def get_pretty_xml(self):
        dom = parseString(self.get_xml())
        return dom.toprettyxml()


class OrderStatusResponse(BaseResponse):
    """
    <ClearSale>
        <Orders>
            <Order>
                <ID>TEST-AA11</ID>
                <Status>AMA</Status>
                <Score>22.8200</Score>
            </Order>
        </Orders>
    </ClearSale>
    """


class SendOrdersResponse(BaseResponse):
    """
    <ClearSale>
        <PackageStatus>
            <TransactionID></TransactionID>
            <StatusCode></StatusCode>
            <Message></Message>
            <Orders>
                <Order>
                    <ID></ID>
                    <Status></Status>
                    <Score></Score>
                </Order>
            </Orders>
        </PackageStatus>
    </ClearSale>
    """



from distutils.core import setup
from setuptools import find_packages

VERSION = __import__("clearsale").__version__

CLASSIFIERS = [
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Topic :: Software Development',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2'
]

install_requires = [
    'suds==0.4',
    'xmltodict==0.9.2'
]

setup(
    name="clearsale",
    packages=find_packages(exclude=["tests"]),
    description="Python library to use ClearSale Total Anti Fraud",
    version=VERSION,
    author="Mateus Vanzo de Padua",
    author_email="mateuspaduaweb@gmail.com",
    license='MIT License',
    platforms=['OS Independent'],
    url="https://github.com/mateuspadua/clearsale-python",
    keywords=['clearsale', 'antifraude', 'anti', 'fraude', 'python'],
    include_package_data=True,
    install_requires=install_requires,
    classifiers=CLASSIFIERS,
)
